@@ -363,7 +363,7 @@ class AlignNR:
             self._plt_im = ax0.matshow(imi)
             
             # quiver
-            dyxr = (dy**2+dx**2)**0.5
+            dyxr = np.hypot(dy, dx)
             shape = dy.shape
             
             s = np.ceil(np.product(shape)**0.5 / 24).astype(int)
