@@ -42,10 +42,10 @@ except:
 class DPC_Explorer:
     def __init__(self, d, r_min=0, r_max=None, r_min_pct=0, r_max_pct=95,
                  descan=[0, 0, 0, 0], cyx=None, vectrot=0, gaus=0, pct=None, 
-                 dt=0, gaus_lim=16, cw_rmin=1.0/3, cmap=None, median=False,
-                 median_mode=0, ransac=False, ransac_dict=None, nbins=None,
-                 hist_lims=None,flip_y=False, flip_x=False, origin='top',
-                 yx_range_from_r=True):
+                 dt=0, gaus_lim=16, cw_rmin=1.0/3, cmap=None, cmap_nfold=1,
+                 median=False, median_mode=0, ransac=False, ransac_dict=None,
+                 nbins=None, hist_lims=None,flip_y=False, flip_x=False,
+                 origin='top', yx_range_from_r=True):
         '''
         Interactive plots of vector field using matplotlib.
         
@@ -93,7 +93,9 @@ class DPC_Explorer:
         cmap : mpl.colors.Colormap, string, or integer
             Colour map for r-theta plot.
             If string, must be name of matplotlib.Colormap.
-            If integer, selects n'th built-in cmaps. 
+            If integer, selects n'th built-in cmaps.
+        cmap_nfold : int
+            The n-fold rotational symmetry of the colour map.
         median : bool
             If True, apply median line correction to data vertically.
             This assumes scan is sequence of horizontal lines.
@@ -223,7 +225,7 @@ class DPC_Explorer:
         
         self._origin = origin.lower()
         if isinstance(d, int):
-            dim = np.indices((np.abs(d),)*2).astype(float)
+            dim = np.indices((np.abs(d),)*2, dtype=np.float)
             dim -= dim.mean((1, 2))[:, None, None]
             dim *= np.array([-1, 1])[:, None, None]      # flip y
             if d < 0:
@@ -274,9 +276,9 @@ class DPC_Explorer:
             y_pct = pct[0]
             x_pct = pct[1]
             if isinstance(y_pct, (int, float)):
-                y_pct = (pct,)*2
+                y_pct = (y_pct,)*2
             if isinstance(x_pct, (int, float)):
-                x_pct = (pct,)*2
+                x_pct = (x_pct,)*2
             self._pct = (y_pct, x_pct)
         else:
             self._pct = ((0,)*2,)*2
@@ -305,7 +307,8 @@ class DPC_Explorer:
         self._xy_ims = None
         
         # CMAP
-        self._init_cmap(cmap)
+        self._init_cmap(cmap, cmap_nfold)
+        self._cmap_nfold = cmap_nfold
         
         # radius theta
         self.r = None
@@ -356,10 +359,10 @@ class DPC_Explorer:
         
     # scanY, scanX
     def _make_scan_indices(self):
-        self._inds = np.indices(self._y_orig.shape).astype(float)
+        self._inds = np.indices(self._y_orig.shape, dtype=np.float)
         self._inds -= self._inds.mean((1, 2))[:, None, None]       # centered
         
-    def _init_cmap(self, cmap_in):
+    def _init_cmap(self, cmap_in, cmap_nfold):
         self._cmaps = collections.OrderedDict()     # dict of all cmaps
         
         # user supplied
@@ -423,6 +426,13 @@ class DPC_Explorer:
             print("WARNING: Perceptually uniform cmap MPL not found.")
             pass
         
+        # set nfold
+        if cmap_nfold != 1:
+            for cmap_ki, cmap_vi in self._cmaps.items():
+                n = max(512, cmap_vi.N * cmap_nfold)
+                cmap_vi = cmap_vi(np.linspace(0, 1, n)) 
+                self._cmaps[cmap_ki] = mpl.colors.ListedColormap(np.vstack((cmap_vi,) * cmap_nfold))
+        
         # set current cmap
         if isinstance(cmap_in, numbers.Integral):
             # to cmapth entry
@@ -488,7 +498,7 @@ class DPC_Explorer:
         # vector rotation
         def rotvec(y, x, deg):
             # rotate vector anticlockwise by angle deg (>0)
-            r = np.sqrt(y**2 + x**2)
+            r = np.hypot(y, x)
             t = np.arctan2(y, x)
             t2 = t+np.deg2rad(deg)
             y2 = r*np.sin(t2)
@@ -637,7 +647,7 @@ class DPC_Explorer:
         cy = self.y-self._yc
         cx = self.x-self._xc
 
-        self.r = np.sqrt(cy**2 + cx**2)
+        self.r = np.hypot(cy, cx)
         self._t = np.arctan2(cy, cx)
         # applied angle offset to plot data
         tplot = self._t-self._dt
@@ -762,10 +772,23 @@ class DPC_Explorer:
                                              lw=1.25)
         
         #rwh = (self._xedges[1]-self._xedges[0])*2
-        trans = ax.transData.inverted()
-        rwh = (trans.transform(ax.transAxes.transform((0.02,)*2))
-               - np.array(ax.axis())[0::2])
-        rwh = rwh[0]
+        
+        #trans = ax.transData.inverted()
+        #rwh = (trans.transform(ax.transAxes.transform((0.02,)*2))
+               #- np.array(ax.axis())[0::2])
+        #rwh = rwh[0]
+        
+        ## above makes large square for small data ranges
+        ## below does the same, so issue probably lies with taking plot range at this stage
+        #dpix = ax.transAxes.transform((0.02,)*2) - ax.transAxes.transform((0.0,)*2)
+        #dpix = max([np.ceil(np.max(dpix)), 5])
+        #trans = ax.transData.inverted()
+        #dxy = trans.transform((dpix,)*2) - trans.transform((0.0,)*2)
+        #rwh = dxy.max()
+        
+        # simply set using data range
+        rwh = self._xedges.ptp() * 0.04
+        
         self._rect = mpl.patches.Rectangle((self._xc-rwh/2, self._yc-rwh/2),
                                           rwh, 
                                           rwh,
@@ -1020,7 +1043,7 @@ class DPC_Explorer:
                 mins = mins[..., None]
                 maxs = maxs[..., None]
             ims = (ims - mins) / (maxs - mins) * 255   # all [0, 255]
-            ims = ims.astype('u1')
+            ims = ims.astype('u1', copy=False)
         ims = ims.squeeze()
         
         if self._seq:
@@ -1331,7 +1354,34 @@ class DPC_Explorer:
         
     def _on_print(self, event):
         # print command options for current settings to stdout
-        cmd_str = "r_min=%0.4f, r_max=%0.4f, descan=[%0.4f, %0.4f, %0.4f, %0.4f], cyx=[%0.4f, %0.4f], vectrot=%0.2f, gaus=%0.2f, pct=[[%0.3f, %0.3f], [%0.3f, %0.3f]], dt=%0.2f, median=%s, median_mode=%d, ransac=%s, ransac_dict=%s, flip_y=%s, flip_x=%s, origin=%s, yx_range_from_r=%s" %(self._r_min, self._r_max, self._descanY_factY*1000, self._descanY_factX*1000, self._descanX_factY*1000, self._descanX_factX*1000, self._yc, self._xc, self._vectrot, self._gaus, self._pct[0][0], self._pct[0][1], self._pct[1][0], self._pct[1][1], self._dt, str(self._median), self._median_mode, str(self._ransac), str(self._ransac_dict), str(self._flip_y), str(self._flip_x), self._origin, str(self._yx_range_from_r))
+        cmd_str = []
+        
+        cmd_str.append("r_min={:0.4f}".format(self._r_min))
+        cmd_str.append("r_max={:0.4f}".format(self._r_max))
+        cmd_str.append("descan=[{:0.4f}, {:0.4f}, {:0.4f}, {:0.4f}]".format(self._descanY_factY*1000,
+                                                                            self._descanY_factX*1000,
+                                                                            self._descanX_factY*1000,
+                                                                            self._descanX_factX*1000))
+        cmd_str.append("cyx=[{:0.4f}, {:0.4f}]".format(self._yc, self._xc))
+        cmd_str.append("vectrot={:0.2f}".format(self._vectrot))
+        cmd_str.append("gaus={:0.2f}".format(self._gaus))
+        cmd_str.append("pct=[[{:0.3f}, {:0.3f}], [{:0.3f}, {:0.3f}]]".format(self._pct[0][0],
+                                                                             self._pct[0][1],
+                                                                             self._pct[1][0],
+                                                                             self._pct[1][1]))
+        cmd_str.append("dt={:0.2f}".format(self._dt))
+        cmd_str.append("cmap={:d}".format(list(self._cmaps.keys()).index(self._cmap_curent[0])))
+        cmd_str.append("cmap_nfold={:d}".format(self._cmap_nfold))
+        cmd_str.append("median={:s}".format(str(self._median)))
+        cmd_str.append("median_mode={:d}".format(self._median_mode))
+        cmd_str.append("ransac={:s}".format(str(self._ransac)))
+        cmd_str.append("ransac_dict={:s}".format(str(self._ransac_dict)))
+        cmd_str.append("flip_y={:s}".format(str(self._flip_y)))
+        cmd_str.append("flip_x={:s}".format(str(self._flip_x)))
+        cmd_str.append("origin={:s}".format(str(self._origin)))
+        cmd_str.append("yx_range_from_r={:s}".format(str(self._yx_range_from_r)))
+
+        cmd_str = ', '.join(cmd_str)
         
         print(cmd_str)
         #print('\n')
@@ -1480,9 +1530,9 @@ class DPC_Explorer:
         else:
             # must be in circle
             #print('x0=%f, xpress=%f, event.xdata=%f, dx=%f, x0+dx=%f'%(x0, xpress, event.xdata, dx, x0+dx))
-            r0 = ((xpress-x0)**2+(ypress-y0)**2)**0.5
+            r0 = np.hypot(xpress-x0, ypress-y0)
             t0 = np.arctan2(ypress-y0, xpress-x0)
-            r = ((event.xdata-x0)**2+(event.ydata-y0)**2)**0.5
+            r = np.hypot(event.xdata-x0, event.ydata-y0)
             t = np.arctan2(event.ydata-y0, event.xdata-x0)
             dt = -(t-t0)
             if self._ctrl:
