@@ -1,6 +1,7 @@
 
 import matplotlib.pylab as plt
 import numpy as np
+import scipy as sp
 
 from scipy.ndimage import gaussian_filter, gaussian_filter1d
 from matplotlib.widgets import Slider
@@ -115,11 +116,11 @@ def im2fftrdf(image, dy=1, dx=1, spf=1, pad_to='square'):
     xf = fft_module.fftfreq(cols, dx)
     
     yyf, xxf = np.meshgrid(yf, xf, indexing='ij')
-    rrf = (yyf**2 + xxf**2)**0.5
+    rrf = np.hypot(yyf, xxf)
     
     # scale allows us to use bincounting
     # set for ~1.0 pixel resolution
-    r_scale = 1.0/rrf.max() * (rows**2 + cols**2)**0.5 / 2.0 * spf
+    r_scale = 1.0/rrf.max() * np.hypot(rows, cols) / 2.0 * spf
     rrf = rrf * r_scale 
     rrf_int = rrf.round(0).astype(int)
     
@@ -469,7 +470,7 @@ class BandpassPlotter:
             row = int(y + 0.5)
             if col>=0 and col<numcols and row>=0 and row<numrows:
                 z = self.im_fft[row,col]
-                r = (x**2+y**2)**0.5
+                r = np.hypot(x, y)
                 return 'x=%0.4f, y=%0.4f, r=%0.4f'%(x, y, r)
             else:
                 return 'x=%1.4f, y=%1.4f' %(x, y)
@@ -811,7 +812,7 @@ def cepstrum2(image, mode='real', sigma=1, pad_to=None, plot=False):
 
     # scale and add background
     yi, xi = np.indices(im_shape)
-    ri = ((xi - xi.mean())**2 + (yi - yi.mean())**2)**0.5
+    ri = np.hypot(xi - xi.mean(), yi - yi.mean())
     s = -(ri - ri.max())
     s /= s.max()
     im = im * s + s*20
@@ -852,7 +853,7 @@ def cepstrum2(image, mode='real', sigma=1, pad_to=None, plot=False):
         def format_coord(x, y):
             x = x - cyx[1]
             y = y - cyx[0]
-            r = (x**2 + y**2)**0.5
+            r = np.hypot(x, y)
             t = np.arctan2(y, x)
             td = np.rad2deg(t)
             return 'x=%1.1f, y=%1.1f, r=%1.1f, t=%1.1f' % (x, y, r, td)
@@ -911,3 +912,54 @@ def cepstrum(a, mode='real', sigma=1, plot=False):
         plt.semilogy(cp)
     return cp
 
+
+def fftcorrelate(im1, im2, phase_exponent=0):
+    '''
+    Correlate two images using FFTs.
+
+    Parameters
+    ----------
+    im1 : 2-D array
+        First real image.
+    im2 : 2-D array
+        Second real image. This can be smaller than `im1`, but should be
+        have the relevant feature centred within its limits if the output
+        is to have peaks at the locations of the equivalent features in `im`.
+        Otherwise, the offset between features is relative to the centre of
+        the image.
+    phase_exponent : scalar in [0, 1]
+        Exponent used in the normalisation of the correlation.
+        0 : cross-correlation
+        1 : phase-correlation
+
+    Returns
+    -------
+    c : 2D array
+        The result of correlation of `im1` and `im2`. The array is the
+        same size as `im1` and centered with respect to the equivalent full
+        output, matching the behaviour of scipy.signal.fftconvolve with 
+        `mode='same'`.
+    
+    '''
+    
+    im2 = im2[::-1, ::-1]
+    
+    s1 = im1.shape
+    s2 = im2.shape
+    shape = [s1[i] + s2[i] - 1 for i in range(im1.ndim)]
+    
+    # ffts
+    fshape = [fft_module.next_fast_len(shapei, True) for shapei in shape]
+    f1 = fft_module.fft2(im1, fshape)
+    f2 = fft_module.fft2(im2, fshape)
+    c = f1 * f2
+    if phase_exponent != 0:
+        c /= np.abs(c) ** phase_exponent + np.finfo(np.float32).eps
+    c = np.abs(fft_module.ifft2(c, fshape))
+    
+    # crops
+    fslice = tuple([slice(sz) for sz in shape])
+    c = c[fslice]
+    c = sp.signal.signaltools._centered(c, s1).copy()
+    return c
+
