@@ -1150,7 +1150,10 @@ def tie(im_grad, im0, ypix=1e-9, xpix=1e-9, kV=200.0):
 def tem_dpc(dyx, df, thickness, pix_m, kV=200):
     '''
     Induction mapping using the phase recovered from Fresnel imaging
-    using the TEM-DPC method.
+    using the TEM-DPC method. [1, 2]
+    
+    [1] Microsc. Microanal. 27, 1113 (2021) https://doi.org/10.1017/S1431927621012551
+    [2] Microsc. Microanal. 27, 1123 (2021) https://doi.org/10.1017/S1431927621012575
     
     Parameters
     ----------
