@@ -13,6 +13,8 @@ import multiprocessing as mp
 from functools import partial, reduce
 from threadpoolctl import threadpool_limits
 
+from scipy.ndimage import shift
+
 from .fpd_processing import rebinA
 
 plt.ion()
@@ -167,7 +169,7 @@ def disk_image(intensity=None, dose=None, radius=32, sigma=1.0, size=256, upscal
     elif np.issubdtype(dtype, np.float):
         dtype_max = np.finfo(dtype).max
     assert(disk.max() <= dtype_max)
-    disk = disk.astype(dtype)
+    disk = disk.astype(dtype, copy=False)
     return disk
 
 
@@ -242,7 +244,7 @@ def shift_array(scan_len=32, shift_min=-8.0, shift_max=8.0, shift_type=0):
     >>> import numpy as np
     
     >>> shiftyy, shiftxx = synthetic_data.shift_array(shift_type=0)
-    >>> shift_mag = np.sqrt(shiftyy**2+shiftxx**2)
+    >>> shift_mag = np.hypot(shiftyy, shiftxx)
     >>> shift_deg = np.rad2deg(np.arctan2(shiftyy, shiftxx))
 
     >>> f, ((ax1, ax2), (ax3, ax4)) = plt.subplots(2, 2, sharex=True, sharey=True, figsize=(8,8))
@@ -279,6 +281,45 @@ def shift_array(scan_len=32, shift_min=-8.0, shift_max=8.0, shift_type=0):
     return(shiftyy, shiftxx)
 
 
+def _fill_shifted_image(im, dyx, fill_value):
+    '''
+    Fill missing pixels of shifted image in-place.
+    
+    dyx can be a single set of value or multiple values ([n,] y, x).
+    
+    Always only 1 image.
+    
+    '''
+    
+    if dyx is not None:
+        dyx = np.asarray(dyx)
+        if dyx.ndim > 1:
+            # multiple dyx values 
+            dyx_min = dyx.min((-2, -1))
+            dyx_max = dyx.max((-2, -1))
+        else:
+            dyx_min = dyx_max = dyx
+        dyx_min = np.floor(dyx_min).astype(int)
+        dyx_max = np.ceil(dyx_max).astype(int)
+        
+        dy2, dx2 = dyx_min
+        dy1, dx1 = dyx_max
+        
+        # handle -ve / +ve here (coerce to 0 or axis size)
+        dx1 = max(dx1, 0)
+        dy1 = max(dy1, 0)
+        if dx2 >= 0:
+            dx2 = im.shape[1]
+        if dy2 >= 0:
+            dy2 = im.shape[0]
+        
+        im[..., :dy1, :] = fill_value
+        im[..., dy2:, :] = fill_value
+        im[..., :dx1] = fill_value
+        im[..., dx2:] = fill_value
+    return None
+
+
 def shift_im(im, dyx, noise=False, method='linear', fill_value=0):
     '''
     Shift image `im` by amount in `dyx`, a tuple of (dy, dx).
@@ -293,15 +334,15 @@ def shift_im(im, dyx, noise=False, method='linear', fill_value=0):
     noise : bool
         If True, shifted image has Poissonian noise.
     method : string
-        One of 'linear' or 'fourier'. Determines method for image
-        shifting. If 'linear', scipy.interpolate.RegularGridInterpolator
-        is used and `fill_value` determines the behaviour for extrapolated
-        points. If 'fourier', the image is shifted cyclically and `fill_value`
-        is not used.
+        Method for image shifting. One of ['linear', 'fourier', 'pixel'].
+        If 'linear', a bi-linear method is used.
+        If 'fourier', the image is shifted cyclically.
+        If 'pixel', the data is shifted with pixel resolution.
+        In all cases, `fill_value` is used to replace extrapolated points.
     fill_value : scalar or None
-        The value to use for points outside of the interpolation
-        domain. If None, values outside the domain are extrapolated.
-        Only used for `method='linear'`.
+        The value to use for points outside of the interpolation domain.
+        If None and `method='linear'`, values outside the domain are
+        extrapolated. Otherwise, zero is used.
     
     Returns
     -------
@@ -310,31 +351,29 @@ def shift_im(im, dyx, noise=False, method='linear', fill_value=0):
     
     '''
     
-    methods = ['linear', 'fourier']
+    methods = ['linear', 'fourier', 'pixel']
     method = method.lower()
     if method not in methods:
-        raise NotImplementedError('interpolation method is unknown')
+        raise Exception("'method' (%s) must be one of: " %(method), methods)
+    
+    if method != 'linear' and fill_value is None:
+        fill_value = 0
     
     # add processing of multiple dyxs and parallel processing?
     if method == 'fourier':
         im_new = fourier_shift(np.fft.fftn(im), dyx)
         im_new = np.abs(np.fft.ifftn(im_new))
     elif  method == 'linear':
-        from scipy.interpolate import RegularGridInterpolator
-        
-        yy, xx = np.indices(im.shape)
-        y = yy[:, 0]
-        x = xx[0, :]
-        
-        f_interp = RegularGridInterpolator((y, x), im, method='linear', bounds_error=False, fill_value=fill_value)
-        
-        yyi = yy - dyx[0]
-        xxi = xx - dyx[1]
-
-        im_new = f_interp((yyi, xxi))
+        im_new = shift(im, shift=dyx, order=1)
+    elif method == 'pixel':
+        dyx = np.round(dyx, 0).astype(int)
+        im_new = np.roll(im, dyx, axis=(-2, -1))
+    
+    _fill_shifted_image(im_new, dyx, fill_value)
     
     if noise:
-        im_new = np.random.poisson(im_new).astype('u2')
+        im_new = np.random.poisson(im_new).astype('u2', copy=False)
+    
     return im_new
 
 
@@ -424,15 +463,15 @@ def shift_images(shifts, image, noise=False, dtype=None, parallel=True, ncores=N
         Controls y-origin of returned data. If origin='top', pythonic indexing 
         is used. If origin='bottom', increasing y is up.
     method : string
-        One of 'linear' or 'fourier'. Determines method for image
-        shifting. If 'linear', scipy.interpolate.RegularGridInterpolator
-        is used and `fill_value` determines the behaviour for extrapolated
-        points. If 'fourier', the image is shifted cyclically and `fill_value`
-        is not used.
+        Method for image shifting. One of ['linear', 'fourier', 'pixel'].
+        If 'linear', a bi-linear method is used.
+        If 'fourier', the image is shifted cyclically.
+        If 'pixel', the data is shifted with pixel resolution.
+        In all cases, `fill_value` is used to replace extrapolated points.
     fill_value : scalar or None
-        The value to use for points outside of the interpolation
-        domain. If None, values outside the domain are extrapolated.
-        Only used for `method='linear'`.
+        The value to use for points outside of the interpolation domain.
+        If None and `method='linear'`, values outside the domain are
+        extrapolated. Otherwise, zero is used.
     
     Returns
     -------
@@ -491,7 +530,7 @@ def shift_images(shifts, image, noise=False, dtype=None, parallel=True, ncores=N
     elif np.issubdtype(target_dtype, np.float):
         dtype_max = np.finfo(target_dtype).max
     assert shifted_ims.max() <= dtype_max
-    shifted_ims = shifted_ims.astype(target_dtype)
+    shifted_ims = shifted_ims.astype(target_dtype, copy=False)
     return shifted_ims
 
 
@@ -559,7 +598,7 @@ def segmented_detectors(im_shape=(256,)*2, rio=(28, 64), cyx=None,
     rin, rout = rio
     
     yi, xi = np.indices(im_shape)
-    ri = ((yi-cy)**2 + (xi-cx)**2)**0.5
+    ri = np.hypot(yi-cy, xi-cx)
 
     tr = np.logical_and(yi <= cy, xi > cx)
     tl = np.logical_and(yi <= cy, xi <= cx)
