@@ -8,13 +8,12 @@ from skimage.feature import peak_local_max
 from skimage.morphology import erosion, disk
 from scipy.spatial import cKDTree
 from scipy.optimize import minimize
-from scipy.signal import fftconvolve
 import warnings
 
 from .fpd_processing import rotate_vector
 from .utils import gaussian_2d_peak_fit
 from .AlignNR_class import AlignNR
-from .fft_tools import im2fftrdf
+from .fft_tools import im2fftrdf, fftcorrelate
 
 import matplotlib.pylab as plt
 plt.ion()
@@ -779,7 +778,7 @@ def orb_trans(im_ref, im_new, pct=0.1, fminmax=None, gy=2, gaus=None, gaus_der=T
             im = gaussian_filter(im, gaus, g_order)
         vmin, vmax = np.percentile(im, [pct, 100-pct])
         im = (im - vmin)/(vmax - vmin)
-        im = (im.clip(0, 1)*255).astype(np.uint8)
+        im = (im.clip(0, 1)*255).astype(np.uint8, copy=False)
         return im
     
     # fft filter
@@ -1167,7 +1166,7 @@ def optimise_trans(im_ref, imw, trans, roi_s=None, print_stats=True):
     return trans_opt
 
 
-def blob_log_detect(image, min_radius, max_radius, num_radii, threshold, overlap=0.3, log_scale=True, ref_im=None, sigma=2.0, log_xc_max_r=None, subpix_log=False, subpix_xc=False, subpix_dict=None, fit_hw=2, plot=False):
+def blob_log_detect(image, min_radius, max_radius, num_radii, threshold, overlap=0.3, log_scale=True, ref_im=None, sigma=2.0, phase_exponent=0, log_xc_max_r=None, subpix_log=False, subpix_xc=False, subpix_dict=None, fit_hw=2, plot=False):
     '''
     Detect blobs in an image using the Laplacian of Gaussian optionally
     combined with edge filtered cross-correlation.
@@ -1191,10 +1190,22 @@ def blob_log_detect(image, min_radius, max_radius, num_radii, threshold, overlap
         If True, intermediate radii are set on a log10 scale. Otherwise,
         linear scaling is used.
     ref_im : None or ndarray
-        If not None, a 2-D array used for cross-correlation of image
-        derivatives. Derivatives are calculated using `fpd.utils.gaus_im_div1d`. 
+        If not None, a 2-D array used for correlation of images. This image can be
+        smaller than the data being processed, but it should be centred for locations
+        of peaks in the correlation to match the position in `image`. Otherwise, the
+        offset of the peak in the correlation image from the centre of the image
+        indicates the displacement between images. See `sigma` for details on how
+        `ref_im` and `image` are processed.
     sigma : scalar
-        Width of Gaussian used for image derivatives.
+        Controls how `ref_im` and `image` are processed. If positive, it is the standard
+        deviation of the Gaussian used for image derivatives of `ref_im` and `image`. If
+        set to 0, both `ref_im` and `image` are passed through unaffected. If negative,
+        `ref_im` is modified by subtracting from itself `ref_im` smoothed by a Gaussian
+        of this standard deviation.
+    phase_exponent : scalar in [0, 1]
+        Exponent used in the normalisation of the correlation.
+        0 : cross-correlation
+        1 : phase-correlation
     log_xc_max_r : None or scalar
         The maximum distance between log and cross-correlation coordinated for the
         cross-correlation peaks to be kept. If None, the average of `min_radius`
@@ -1217,12 +1228,20 @@ def blob_log_detect(image, min_radius, max_radius, num_radii, threshold, overlap
     Returns
     -------
     blobs : ndarray
-        N x 4 array, where N is the number of blobs detected. The second axis
-        is y, x, r, a where 'r' is the radius and 'a' is a measure of the signal:
-        if ref_im is None and not subpix_log: image value at blob centre.
-        if ref_im is None and subpix_log: fitted image value at blob centre.
-        if ref_im is not None and not subpix_xc: cross-correlation value at blob centre.
-        if ref_im is not None and subpix_xc: fitted cross-correlation value at blob centre.
+        N x 6 array, where N is the number of blobs detected. The second axis
+        is `y, x, r, a, dy, dx` where `r` is the radius, `a` is a measure of the signal,
+        and `dy, dx` is the error in `y, x`, where available.
+        
+        For the signal, `a`:
+            if ref_im is None and not subpix_log: image value at blob centre.
+            if ref_im is None and subpix_log: fitted image value at blob centre.
+            if ref_im is not None and not subpix_xc: cross-correlation value at blob centre.
+            if ref_im is not None and subpix_xc: fitted cross-correlation value at blob centre.
+        For the error estimates, `dy, dx`:
+            if ref_im is None and not subpix_log: nan, nan.
+            if ref_im is None and subpix_log: error values from the fit.
+            if ref_im is not None and not subpix_xc: nan, nan.
+            if ref_im is not None and subpix_xc: error values from the fit.
         
     Notes
     -----
@@ -1230,7 +1249,7 @@ def blob_log_detect(image, min_radius, max_radius, num_radii, threshold, overlap
     
     See Also
     --------
-    background_erosion, friedel_filter, vector_combinations, lattice_angles, lattice_magnitudes, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2
+    background_erosion, friedel_filter, vector_combinations, lattice_angles, lattice_magnitudes, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2, strain
 
     Notes
     -----
@@ -1238,9 +1257,17 @@ def blob_log_detect(image, min_radius, max_radius, num_radii, threshold, overlap
     
     '''
     
+    # TODO switch to namedtuple so more values can be easily returned?
+    
     if ref_im is not None:
-        from .utils import gaus_im_div1d
-        ref_im_div = gaus_im_div1d(ref_im, sigma)
+        if sigma > 0:
+            from .utils import gaus_im_div1d
+            ref_im_div = gaus_im_div1d(ref_im, sigma)
+        elif sigma < 0:
+            ref_im = ref_im.astype(float, copy=False)
+            ref_im_div = ref_im - gaussian_filter(ref_im, np.abs(sigma))
+        else:
+            ref_im_div = ref_im
         if plot:
             plt.matshow(ref_im_div)
 
@@ -1270,7 +1297,6 @@ def blob_log_detect(image, min_radius, max_radius, num_radii, threshold, overlap
     yi, xi = blobs_log.round(0).astype(int)[:, [0, 1]].T
     blobs_log[:, 3] = image[yi, xi]
     
-    
     ### peak-fit directly
     if subpix_log:
         if fit_hw is None:
@@ -1287,12 +1313,17 @@ def blob_log_detect(image, min_radius, max_radius, num_radii, threshold, overlap
                 'plot' : False}
         if subpix_dict is not None:
             spd.update(subpix_dict)
-        popt, _ = gaussian_2d_peak_fit(image=image, **spd)
+        popt, perr = gaussian_2d_peak_fit(image=image, **spd)
         good_fit = np.isfinite(popt[:, 0])
         # recheck within bounds or check if <1pix?
         blobs_log = np.column_stack((popt[good_fit, 1:3],               # yx
                                      blobs_log[good_fit, 2],            # r
-                                     popt[good_fit, 0] + popt[good_fit, -1]))   # offset + amplitude
+                                     popt[good_fit, 0] + popt[good_fit, -1],   # offset + amplitude
+                                     perr[good_fit, 1:3]))               # error in yx
+    else:
+        dydx = np.empty_like(blobs_log[:, :2])
+        dydx[:] = np.nan
+        blobs_log = np.column_stack((blobs_log, dydx))
     
     
     ### use pattern matching
@@ -1301,26 +1332,44 @@ def blob_log_detect(image, min_radius, max_radius, num_radii, threshold, overlap
         image_log[image_log==0] = 1
         image_log = np.log(image_log)
     
-        image_div = gaus_im_div1d(image_log, sigma)
-        c = fftconvolve(image_div, ref_im_div, mode='same')
-        pks = peak_local_max(c, min_distance=1, threshold_abs=None, threshold_rel=None, exclude_border=True).astype(float)
+        if sigma > 0:
+            image_div = gaus_im_div1d(image_log, sigma)
+        else:
+            image_div = image_log
+        c = fftcorrelate(image_div, ref_im_div, phase_exponent=phase_exponent)
+        
+        # find all peaks and then filter with the KDTree below.
+        pks = peak_local_max(c, min_distance=1, threshold_abs=None, threshold_rel=None, exclude_border=True)
         
         # filter xc peaks by log blobs
-        ct = cKDTree(blobs_log[:, :2])
+        ct = cKDTree(pks)
         if log_xc_max_r is None:
             log_xc_max_r = (min_radius + max_radius)/2.0
-        inds = ct.query_ball_point(x=pks, r=log_xc_max_r)
-        ninds = np.array([len(t) for t in inds])
-        b = ninds == 1
-        blobs_xc = pks[b]
-        # could filter by any repeated peaks by intensity
+        inds = ct.query_ball_point(x=blobs_log[:, :2], r=log_xc_max_r)
         
-        t = np.hstack(inds[b])
+        pks_filt = []
+        t = np.zeros(len(blobs_log), dtype=bool)
+        for i, indsi in enumerate(inds):
+            li = len(indsi)
+            if li == 0:
+                t[i] = False
+                continue
+            elif li == 1:
+                pks_filt.append(pks[indsi[0]])
+                t[i] = True
+            else:
+                # li > 1, filter peaks by intensity
+                yi, xi = pks[indsi].T
+                j = c[yi, xi].argmax()
+                pks_filt.append(pks[indsi[j]])
+                t[i] = True
+        pks_filt = np.array(pks_filt)
+        
+        # build array with yx, r and a measure of amplitude
         r = blobs_log[:, 2][t]
-        yi, xi = blobs_xc.round(0).astype(int)[:, [0, 1]].T
+        yi, xi = pks_filt.round(0).astype(int)[:, [0, 1]].T
         a = c[yi, xi]
-        blobs_xc = np.column_stack((blobs_xc, r, a))
-        
+        blobs_xc = np.column_stack((pks_filt, r, a))
         
         ### do peak-fitting
         if subpix_xc:
@@ -1338,13 +1387,18 @@ def blob_log_detect(image, min_radius, max_radius, num_radii, threshold, overlap
                     'plot' : False}
             if subpix_dict is not None:
                 spd.update(subpix_dict)
-            popt, _ = gaussian_2d_peak_fit(image=c, **spd)
+            popt, perr = gaussian_2d_peak_fit(image=c, **spd)
             good_fit = np.isfinite(popt[:, 0])
             # recheck within bounds or check if <1pix?
             
             blobs_xc = np.column_stack((popt[good_fit, 1:3],               # yx
                                         blobs_xc[good_fit, 2],             # r
-                                        popt[good_fit, 0] + popt[good_fit, -1]))   # offset + amplitude
+                                        popt[good_fit, 0] + popt[good_fit, -1], # offset + amplitude
+                                        perr[good_fit, 1:3]))               # error in yx
+        else:
+            dydx = np.empty_like(blobs_xc[:, :2])
+            dydx[:] = np.nan
+            blobs_xc = np.column_stack((blobs_xc, dydx))
     
     if plot:
         cmap = plt.cm.gray
@@ -1373,7 +1427,8 @@ def blob_log_detect(image, min_radius, max_radius, num_radii, threshold, overlap
         if ref_im is not None:
             plt.matshow(c)
             plt.axis('off')
-            plt.plot(pks[:, 1], pks[:, 0], 'xr')
+            #plt.plot(pks[:, 1], pks[:, 0], 'xr', alpha=0.2)
+            plt.plot(pks_filt[:, 1], pks_filt[:, 0], 'xr')
             plt.plot(blobs_xc[:, 1], blobs_xc[:, 0], 'ko', mfc='none')
             plt.tight_layout()
         
@@ -1439,7 +1494,7 @@ def friedel_filter(blobs, cyx, min_distance_radii_scale=None, min_distance_pad=N
     
     See Also
     --------
-    background_erosion, blob_log_detect, vector_combinations, lattice_angles, lattice_magnitudes, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2
+    background_erosion, blob_log_detect, vector_combinations, lattice_angles, lattice_magnitudes, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2, strain
     
     '''
     
@@ -1591,7 +1646,7 @@ def synthetic_lattice(cyx, ab, angles, reps=None, shape=None, max_r=None, plot=F
     
     See Also
     --------
-    background_erosion, blob_log_detect, friedel_filter, vector_combinations, lattice_angles, lattice_magnitudes, lattice_inlier, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2
+    background_erosion, blob_log_detect, friedel_filter, vector_combinations, lattice_angles, lattice_magnitudes, lattice_inlier, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2, strain
     
     '''
     
@@ -1700,7 +1755,7 @@ def synthetic_lattice(cyx, ab, angles, reps=None, shape=None, max_r=None, plot=F
         def format_coord(x, y):
             x = x - cyx[1]
             y = y - cyx[0]
-            r = (x**2 + y**2)**0.5
+            r = np.hypot(x, y)
             t = np.arctan2(y, x)
             td = np.rad2deg(t)
             return 'x=%1.4f, y=%1.4f, r=%1.4f, t=%1.4f' % (x, y, r, td)
@@ -1730,7 +1785,7 @@ def vector_combinations(blobs, plot=False):
     
     See Also
     --------
-    background_erosion, blob_log_detect, friedel_filter, lattice_angles, lattice_magnitudes, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2
+    background_erosion, blob_log_detect, friedel_filter, lattice_angles, lattice_magnitudes, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2, strain
     
     '''
 
@@ -1814,7 +1869,7 @@ def lattice_angles(rt, nfold=None, bin_deg=1.0, hist_gaus=2.0, weight_hist=True,
     
     See Also
     --------
-    background_erosion, blob_log_detect, friedel_filter, vector_combinations, lattice_magnitudes, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2
+    background_erosion, blob_log_detect, friedel_filter, vector_combinations, lattice_magnitudes, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2, strain
     
     '''
 
@@ -1844,7 +1899,7 @@ def lattice_angles(rt, nfold=None, bin_deg=1.0, hist_gaus=2.0, weight_hist=True,
     
     hist, bin_edges = np.histogram(t, bins=int(360.0/bin_deg), range=(-np.pi, np.pi), weights=weights)
     bin_cnts = (bin_edges[:-1] + bin_edges[1:])/2.0
-    hist_sm = gaussian_filter1d(hist.astype(float), hist_gaus, mode='wrap')
+    hist_sm = gaussian_filter1d(hist.astype(float, copy=False), hist_gaus, mode='wrap')
     
     # find peaks
     if nfold !=2:
@@ -1919,7 +1974,7 @@ def lattice_magnitudes(rt, angles, window_deg=5, bin_pix=1.0, hist_gaus=2.0, min
     
     See Also
     --------
-    background_erosion, blob_log_detect, friedel_filter, vector_combinations, lattice_angles, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2
+    background_erosion, blob_log_detect, friedel_filter, vector_combinations, lattice_angles, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2, strain
     
     '''
     
@@ -1952,7 +2007,7 @@ def lattice_magnitudes(rt, angles, window_deg=5, bin_pix=1.0, hist_gaus=2.0, min
         
         bin_width = bin_edges[1] - bin_edges[0]
         bin_cnts = (bin_edges[:-1] + bin_edges[1:])/2.0
-        hist_sm = gaussian_filter1d(hist.astype(float), hist_gaus/bin_pix, mode='wrap')
+        hist_sm = gaussian_filter1d(hist.astype(float, copy=False), hist_gaus/bin_pix, mode='wrap')
         
         # periodicity analysis
         if mode == 'fft':           
@@ -2101,7 +2156,7 @@ def lattice_magnitudes(rt, angles, window_deg=5, bin_pix=1.0, hist_gaus=2.0, min
         
         #hist, bin_edges = np.histogram(th, bins=int(360.0/bin_deg), range=(-np.pi, np.pi), weights=weights)
         #bin_cnts = (bin_edges[:-1] + bin_edges[1:])/2.0
-        #hist_sm = gaussian_filter1d(hist.astype(float), hist_gaus, mode='wrap')
+        #hist_sm = gaussian_filter1d(hist.astype(float, copy=False), hist_gaus, mode='wrap')
         
         #pks = peak_local_max(hist_sm, num_peaks=2, min_distance=min_distance)[:, 0]
         #i1, i2 = pks[0], pks[-1]
@@ -2149,7 +2204,7 @@ def lattice_inlier(yx, yxg, r=4, plot=False):
     
     See Also
     --------
-    background_erosion, blob_log_detect, friedel_filter, vector_combinations, lattice_angles, lattice_magnitudes, synthetic_lattice, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2
+    background_erosion, blob_log_detect, friedel_filter, vector_combinations, lattice_angles, lattice_magnitudes, synthetic_lattice, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2, strain
     
     '''
     
@@ -2215,7 +2270,11 @@ def optimise_lattice(yx, cyx, ab, angles, shape, weights=None, constraints=None,
 
     Returns
     -------
-    tuple of cyx_opt, ab_opt, angles_opt
+    tuple of cyx_opt, ab_opt, angles_opt, error
+    `cyx_opt, ab_opt, angles_opt` are the optimised values of `cyx, ab, angles`.
+    `error` is the quadrature combination of the euclidean distance between the
+    experimental and modelled lattices. Divide by the number of points to get
+    the error per point.
     
     Notes
     -----
@@ -2246,7 +2305,7 @@ def optimise_lattice(yx, cyx, ab, angles, shape, weights=None, constraints=None,
     
     See Also
     --------
-    background_erosion,  blob_log_detect, friedel_filter, vector_combinations, lattice_angles, lattice_magnitudes, synthetic_lattice, lattice_inlier, lattice_resolver
+    background_erosion,  blob_log_detect, friedel_filter, vector_combinations, lattice_angles, lattice_magnitudes, synthetic_lattice, lattice_inlier, lattice_resolver, strain
     
     '''
     
@@ -2278,6 +2337,7 @@ def optimise_lattice(yx, cyx, ab, angles, shape, weights=None, constraints=None,
     res = minimize(fun, x0, args=(yx, shape), constraints=constraints,
                    options=options, **kwd)
     x_opt = res.x
+    error = res.fun
     
     cyx_opt = x_opt[:2]
     angles_opt = x_opt[2:4]
@@ -2295,11 +2355,11 @@ def optimise_lattice(yx, cyx, ab, angles, shape, weights=None, constraints=None,
         plt.plot(x_opt[1], x_opt[0], 'bo', ms=6, mfc='none')
         plt.gca().invert_yaxis()
         plt.gca().set_aspect(1)
-        plt.title('error: %0.3f\n%s' %(res.fun, pstr))
+        plt.title('error: %0.3f\n%s' %(error, pstr))
         plt.xlabel('x (pixels)')
         plt.ylabel('y (pixels)')
     
-    return cyx_opt, ab_opt, angles_opt
+    return cyx_opt, ab_opt, angles_opt, error
 
 
 def lattice_resolver(ab, angles, res_angles, reps=(5, 5), search_dangle=1.0, r_max=None):
@@ -2339,7 +2399,7 @@ def lattice_resolver(ab, angles, res_angles, reps=(5, 5), search_dangle=1.0, r_m
     
     See Also
     --------
-    background_erosion, blob_log_detect, friedel_filter, vector_combinations, lattice_angles, lattice_magnitudes, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_from_inliers, fpd.fft_tools.cepstrum2
+    background_erosion, blob_log_detect, friedel_filter, vector_combinations, lattice_angles, lattice_magnitudes, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_from_inliers, fpd.fft_tools.cepstrum2, strain
     
     '''
     
@@ -2451,7 +2511,7 @@ def lattice_from_inliers(yx, cyx, r=8, max_n=10, r_min=None, r_max=None,
     
     See Also
     --------
-    background_erosion, blob_log_detect, friedel_filter, vector_combinations, lattice_angles, lattice_magnitudes, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_resolver, fpd.fft_tools.cepstrum2
+    background_erosion, blob_log_detect, friedel_filter, vector_combinations, lattice_angles, lattice_magnitudes, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_resolver, fpd.fft_tools.cepstrum2, strain
     
     '''
     
@@ -2706,3 +2766,177 @@ def nc_correct(im, nc, plot=False):
     
     return cim
 
+
+def strain(r_ref, t_ref, r, t, invert_space=False, origin='top',
+           plot=True, pct=0, centred_cmap=True):
+    '''
+    Calculate infinitesimal strain tensor of polar coordinates (r, t)
+    using reference coordinates (r_ref, t_ref).
+    
+    Parameters
+    ----------
+    r_ref : length 2 iterable
+        Reference vector magnitudes. If data is in reciprocal space,
+        ensure `invert_space=True` to obtain real-space data.
+    t_ref : length 2 iterable
+        Reference vector angles, in radians. See `origin`.
+    r : ndarray
+        Data vector magnitudes. If data is in reciprocal space, ensure
+        `invert_space=True` to obtain real-space data. The data may be:
+        - 1-D of length 2 for a single point.
+        - 2-D with shape 2xM for a line profile.
+        - 3-D with shape 2xMxN for a y- and x-axes of size M and N.
+    t : ndarray
+        Data vector angles, in radian. The shape should match that of
+        `r`. See `origin`.
+    invert_space : bool
+        If True, the input space in inverted before the strain is
+        calculated.
+    origin : str
+        One of ['top', 'bottom'], determining the y-axis origin of the
+        input data. The output data always has the y-axis origin at
+        the bottom.
+    plot : boolean
+        If True and `r` and `t` do not represent a single data point,
+        the results are plotted. For surface plots, the range is set
+        individually in each panel. See `pct` and `centred_cmap` for
+        surface plot control.
+    pct : scalar
+        The percentile in (0, 100) that sets the plotted range in
+        surface plots. The range is [pct, 100-pct].
+    centred_cmap : bool
+        If True, a centred coolwarm color map is used for surface plots.
+        Otherwise, the default colourmap is used and the plotted range
+        is not forced to be symmetric.
+    
+    Returns
+    -------
+    es : ndarray
+        Infinitesimal strain tensor components, of shape [4, ...], where
+        the 4 elements are [exx, eyy, exy, er]. exy is taken as positive
+        when the angle between positive x- and y-axes becomes more acute.
+        The rotation, er, is positive for anticlockwise rotation (always
+        with the y-axis origin at the bottom, regardless of the `origin`
+        parameter value which only influences interpretation of the input
+        data, not the output data).
+    
+    Notes
+    -----
+    If the reference is spatially resolved, then it should be reduced to
+    representative values by, for example, taking the mean or median of
+    the vector magnitudes and angles.
+    
+    The coordinate system on which the strain is to be assessed may not
+    match the coordinate space that is of interest, such as an interface.
+    Once identified, any misalignment may be corrected by adding appropriate
+    offsets to the angle inputs, `t_ref` and `t`.
+    
+    If the precise offset angle is unknown, say, due to the ideal reference
+    being orthogonal and the real reference being slightly off of orthogonal,
+    then splitting the error across both vectors is a reasonable compromise.
+    
+    See Also
+    --------
+    background_erosion, blob_log_detect, friedel_filter, vector_combinations, lattice_angles, lattice_magnitudes, synthetic_lattice, lattice_inlier, optimise_lattice, lattice_resolver, lattice_from_inliers, fpd.fft_tools.cepstrum2, strain
+    
+    '''
+    
+    # process origin
+    origins = ['top', 'bottom']
+    if origin not in origins:
+        raise Exception("'origin (%s) must be one of: " %(origin), str(origins))
+    
+    if origin=='top':
+        # reverse angles for y pointing up
+        t_ref = -t_ref
+        t = -t
+    
+    # ensure arrays
+    r_ref = np.array(r_ref)
+    t_ref = np.array(t_ref)
+    r = np.array(r)
+    t = np.array(t)
+    
+    # get dimension and shape
+    nd = r.ndim - 1
+    os = r.shape
+    
+    # make 3-D by default
+    for _ in range(3-len(os)):
+        r = r[..., None]
+        t = t[..., None]
+    
+    # generate output array
+    es = np.empty((4,) + r.shape[1:])
+    
+    # calculate ref coordinates
+    xr = r_ref * np.cos(t_ref)
+    yr = r_ref * np.sin(t_ref)
+    ref_coords = np.array([xr, yr]).T
+    if invert_space:
+        ref_coords = np.linalg.inv(ref_coords).T
+    
+    # loop over points
+    for i in range(r.shape[1]):
+        for j in range(r.shape[2]):
+            # calculate coordinates            
+            xi = r[:, i, j] * np.cos(t[:, i, j])
+            yi = r[:, i, j] * np.sin(t[:, i, j])
+            i_coords = np.array([xi, yi]).T
+            if invert_space:
+                try:
+                    i_coords = np.linalg.inv(i_coords).T
+                except np.linalg.LinAlgError:
+                    es[:, i, j] = np.nan
+                    continue
+            
+            # calculate transformation matrix
+            #tm = np.linalg.solve(ref_coords, i_coords)
+            tm, residuals, rank, s = np.linalg.lstsq(ref_coords, i_coords, rcond=None)
+
+            # convert matrix to entry parameters (in notes).
+            t11, t12, t21, t22 = tm.flatten()
+
+            # infinitesimal strain tensor
+            e11 = t11 - 1
+            e12 = (t21 + t12) / 2.0
+            #e21 = e12
+            e22 = t22 - 1
+            er = -(t21 - t12) / 2.0
+            # e12 +ve is more acute
+            
+            es[:, i, j] = e11, e22, e12, er
+    es = np.squeeze(es)
+    
+    if plot:
+        labs = 'exx, eyy, exy, er'.split(', ')
+        if nd == 0:
+            # point
+            pass
+        elif nd == 1:
+            # lines
+            f, axs = plt.subplots(2, 2, sharex=True, sharey=False)
+            for axi, di, labi in zip(axs.flatten(), es, labs):
+                axi.plot(di)
+                axi.set_title(labi)
+            plt.tight_layout()
+        elif nd == 2:
+            # surfaces
+            
+            # limits
+            vmin, vmax = np.nanpercentile(es, [pct, 100-pct], axis=(-2, -1))
+            if centred_cmap:
+                vmin, vmax = (np.nanmax(np.abs([vmin, vmax]), 0)[:, None] * np.array([-1, 1])).T
+                cmap = 'coolwarm'
+            else:
+                cmap = None
+            
+            f, axs = plt.subplots(2, 2, sharex=True, sharey=True)
+            for i, (axi, di, labi) in enumerate(zip(axs.flatten(), es, labs)):
+                imi = axi.matshow(di, cmap=cmap, vmin=vmin[i], vmax=vmax[i])
+                #axi.set_title(labi)
+                plt.colorbar(imi, ax=axi, label=labi)
+            plt.tight_layout()
+    
+    return es
+
